# Copyright 2020 NXP
# SPDX-License-Identifier: BSD-3-Clause
OBJ_CLASSIFICATION = {'image': "cat.jpg",
                      'labels': "labels_mobilenet_quant_v1_224.txt",
                      'model': "mobilenet_v1_1.0_224_quant.tflite",
                      'sha1': "119f412b44429ee54643fd61da54530d844251ca",
                      'src': {'drive': "https://drive.google.com/file/d/"
                                       "1hO9xrXglzoEy0Tv6RsM9YtttTsXNyHws",
                              'github': "https://github.com/NXP/"
                                        "nxp-demo-experience-assets/releases/download/5.10.52_2.1.0/"
                                        "eIQObjectClassificationTFLite.zip"},
                      'window_title': "PyeIQ - Object Classification"}



